@@ -41,11 +41,6 @@
 
 #include <cups/cups.h>
 
-typedef struct {
-	GnomeCupsPrinter *printer;
-	char *value;
-} PrinterInfoData;
-
 static void
 unref_group (gpointer user_data, GObject *obj)
 {
@@ -115,14 +110,26 @@ update_general_page (GladeXML *xml)
 		gnome_cups_printer_get_full_state (printer));
 }
 
+typedef struct {
+	GnomeCupsPrinter *printer;
+	char *value;
+	GtkWidget *entry;
+} EntryHandlerInfo;
+
 static gboolean
-apply_description_handler (gpointer user_data)
+cb_entry_idle_handler (EntryHandlerInfo *data)
 {
-	PrinterInfoData *data = user_data;
+	gchar const *name = gtk_widget_get_name (data->entry);
+
+	g_return_val_if_fail (name != NULL, FALSE);
+
+	if (0 == strcmp (name, "name_entry"))
+		gnome_cups_printer_set_name (data->printer, data->value, NULL);
+	else if (0 == strcmp (name, "description_entry"))
+		gnome_cups_printer_set_description (data->printer, data->value, NULL);
+	else if (0 == strcmp (name, "location_entry"))
+		 gnome_cups_printer_set_location (data->printer, data->value, NULL);
 
-	gnome_cups_printer_set_description (data->printer,
-					    data->value,
-					    NULL);
 	g_object_unref (data->printer);
 	g_free (data->value);
 	g_free (data);
@@ -130,80 +137,31 @@ apply_description_handler (gpointer user_data)
 	return FALSE;
 }
 
-static void
-apply_description (GladeXML *xml, GtkWidget *entry)
+/* You can't hit the main loop in a GtkEntry's focus_out_event, so queue this
+ * in an idle handler */
+static gboolean
+cb_entry_changed (GladeXML *xml, GtkWidget *entry)
 {
-	GnomeCupsPrinter *printer;
-	PrinterInfoData *data;
-
-	/* You can't hit the main loop in a GtkEntry's focus_out_event,
-	 * so queue this in an idle handler */
-
-	data = g_new0 (PrinterInfoData, 1);
-	printer = g_object_get_data (G_OBJECT (xml), "printer");
+	EntryHandlerInfo *data = g_new0 (EntryHandlerInfo, 1);
+	GnomeCupsPrinter *printer = g_object_get_data (G_OBJECT (xml), "printer");
 	g_object_ref (printer);
 	data->printer = printer;
 	data->value = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
-
-	g_idle_add (apply_description_handler, data);
-}
-
-static gboolean
-apply_location_handler (gpointer user_data)
-{
-	PrinterInfoData *data = user_data;
-
-	gnome_cups_printer_set_location (data->printer,
-					 data->value,
-					 NULL);
-	g_object_unref (data->printer);
-	g_free (data->value);
-	g_free (data);
-
+	data->entry = entry;
+	g_idle_add ((GSourceFunc)cb_entry_idle_handler, data);
 	return FALSE;
 }
 
 static void
-apply_location (GladeXML *xml, GtkWidget *entry)
+setup_entry (GladeXML *xml, char const *name)
 {
-	GnomeCupsPrinter *printer;
-	PrinterInfoData *data;
-
-	/* You can't hit the main loop in a GtkEntry's focus_out_event,
-	 * so queue this in an idle handler */
-
-	data = g_new0 (PrinterInfoData, 1);
-	printer = g_object_get_data (G_OBJECT (xml), "printer");
-	g_object_ref (printer);
-	data->printer = printer;
-	data->value = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
-
-	g_idle_add (apply_location_handler, data);
+	GtkWidget *w = glade_xml_get_widget (xml, name);
+	g_signal_connect_swapped (w, "focus_out_event",
+		G_CALLBACK (cb_entry_changed), xml);
+	g_signal_connect (w, "activate",
+		G_CALLBACK (cb_entry_changed), xml);
 }
 
-static gboolean
-description_focus_out_cb (GtkWidget *entry,
-			  GdkEventFocus *event,
-			  gpointer user_data)
-{
-	GladeXML *xml = GLADE_XML (user_data);
-
-	apply_description (xml, entry);
-
-	return FALSE;
-}
-
-static gboolean
-location_focus_out_cb (GtkWidget *entry,
-		       GdkEventFocus *event,
-		       gpointer user_data)
-{
-	GladeXML *xml = GLADE_XML (user_data);
-
-	apply_location (xml, entry);
-
-	return FALSE;
-}
 
 static void
 check_toggled_cb (GtkWidget *widget,
@@ -386,20 +344,12 @@ test_page_clicked_cb (GtkWidget *button, gpointer user_data)
 static void
 setup_general_page (GladeXML *xml)
 {
-	GtkWidget *widget;
-
 	update_general_page (xml);
 
 	if (gnome_cups_can_admin ()) {
-		widget = glade_xml_get_widget (xml, "description_entry");
-		g_signal_connect (widget,
-			"focus_out_event",
-			G_CALLBACK (description_focus_out_cb), xml);
-
-		widget = glade_xml_get_widget (xml, "location_entry");
-		g_signal_connect (widget,
-			"focus_out_event",
-			G_CALLBACK (location_focus_out_cb), xml);
+		setup_entry (xml, "name_entry");
+		setup_entry (xml, "description_entry");
+		setup_entry (xml, "location_entry");
 	}
 
 	hookup_option_menu_xml (xml, "resolution_option_menu", "Resolution");
@@ -637,28 +587,38 @@ cb_connection_changed (GCupsConnectionSelector *cs, GnomeCupsPrinter *printer)
 	}
 }
 
+static void
+cb_display_uri (GnomeCupsPrinter *printer, GCupsConnectionSelector *cs)
+{
+	char const *uri = gnome_cups_printer_get_device_uri (printer);
+	if (uri == NULL)
+		return;
+	gcups_connection_selector_set_uri (cs, uri);
+}
+
 static void
 setup_connection_page (GladeXML *xml, GnomeCupsPrinter *printer)
 {
 	GtkWidget  *w, *cs;
-	char const *uri;
-
 	if (!gnome_cups_can_admin ())
 		return;
 	if (printer == NULL)
 		return;
-	uri = gnome_cups_printer_get_device_uri (printer);
-	if (uri == NULL)
-		return;
 
 	cs = gcups_connection_selector_new ();
 	w = glade_xml_get_widget (xml, "prop_notebook");
 	gtk_notebook_append_page (GTK_NOTEBOOK (w),
 		cs, gtk_label_new (_("Connection")));
-	gcups_connection_selector_set_uri (GCUPS_CONNECTION_SELECTOR (cs),
-		uri);
+	if (gnome_cups_printer_get_attributes_initialized (printer))
+		cb_display_uri (printer, GCUPS_CONNECTION_SELECTOR (cs));
+	else if (NULL == g_object_get_qdata (G_OBJECT (cs), g_quark_from_static_string ("pending"))) {
+		gulong handle = g_signal_connect_object (printer, "attributes-changed",
+			G_CALLBACK (cb_display_uri), cs, 0);
+		g_object_set_qdata (G_OBJECT (cs),
+			g_quark_from_static_string ("pending"),
+			GUINT_TO_POINTER (handle));
+	}
 	gtk_widget_show (cs);
-
 	g_signal_connect_object (cs,
 		"changed",
 		G_CALLBACK (cb_connection_changed), xml, 0);
