@@ -300,9 +300,25 @@ ppd_extract_model_and_driver (GCupsPPD *ppd)
 	if (NULL == without_vendor)
 		without_vendor = (char *)ppd->nickname;
 
+	g_return_if_fail (without_vendor != NULL);
+
+	/* gimp-print ijs models (suse, debian)
+	 * 	<model> Foomatic/gimp-print-ijs */
+	if (NULL != (tmp = strstr (without_vendor, " Foomatic/gimp-print-ijs"))) {
+		model = g_strndup (without_vendor, tmp - without_vendor);
+		driver = g_strdup (_("High Quality Image (GIMP-Print Inkjet)"));
+	}
+
+	/* gimp-print and ijs models (suse, debian)
+	 * 	<model> Foomatic/gimp-print */
+	else if (NULL != (tmp = strstr (without_vendor, " Foomatic/gimp-print"))) {
+		model = g_strndup (without_vendor, tmp - without_vendor);
+		driver = g_strdup (_("High Quality Image (GIMP-Print)"));
+	}
+
         /* Stock foomatic entries (suse, debian)
 	 * 	<model> Foomatic/<driver> */
-	if (NULL != (tmp = strstr (without_vendor, " Foomatic/"))) {
+	else if (NULL != (tmp = strstr (without_vendor, " Foomatic/"))) {
 		model = g_strndup (without_vendor, tmp - without_vendor);
 		driver = g_strdup (tmp + 10);
 
@@ -319,7 +335,8 @@ ppd_extract_model_and_driver (GCupsPPD *ppd)
 	} else if (NULL != (tmp = strstr (without_vendor, " - CUPS+Gimp-Print v"))) {
 		model = g_strndup (without_vendor, tmp - without_vendor);
                 if((NULL != (language_position = strstr (ppd->filename, "/"))) &&
-                   strncmp ("C/", ppd->filename, 2) ) {
+                   strncmp ("C/", ppd->filename, 2) &&
+                   strncmp ("stp/", ppd->filename, 4)) {
                     language = g_strndup (ppd->filename, language_position - ppd->filename);
                     driver = g_strdup_printf (_("High Quality Image (GIMP-Print) (%s)"), language);
                     g_free(language);
@@ -343,7 +360,7 @@ ppd_extract_model_and_driver (GCupsPPD *ppd)
 	}
 
 	/* Lets see if this is a foomatic 'recommended' driver */
-	if (NULL != (tmp = strstr (driver, " (recommended)"))) {
+	if (NULL != (tmp = strstr (without_vendor, " (recommended)"))) {
 		strcpy (tmp, tmp+14);
 		ppd->is_recommended = TRUE;
 	} else
@@ -359,25 +376,31 @@ ppd_extract_model_and_driver (GCupsPPD *ppd)
 static int
 compare_models_by_driver (gconstpointer a, gconstpointer b)
 {
-	const GCupsPPD *a1 = a;
-	const GCupsPPD *b1 = b;
+	GCupsPPD const *a1 = a;
+	GCupsPPD const *b1 = b;
+	if (a1->is_recommended ^ b1->is_recommended)
+		return a1->is_recommended ? -1 : 1;
 	return str_case_compare (a1->driver, b1->driver);
 }
 
 static void
-categorize_ppd (GCupsPPD *ppd, GHashTable *vendors, GladeXML *xml)
+setup_ppd (GCupsDriverSelector *ds, GCupsPPD *ppd, gboolean select)
 {
-	GSList *drivers = NULL, *dup;
-	GHashTable *models = g_hash_table_lookup (vendors, ppd->vendor);
+	GSList *drivers = NULL, *tmp;
+	GHashTable *models;
+
+	ppd_extract_model_and_driver (ppd);
+
+	models = g_hash_table_lookup (ds->vendors, ppd->vendor);
 	if (models == NULL) {
 		models = g_hash_table_new_full (
 			str_case_hash, str_case_equal, NULL,
 			(GDestroyNotify)ppd_list_free);
-		g_hash_table_insert (vendors, ppd->vendor, models);
-		if (xml != NULL) {
+		g_hash_table_insert (ds->vendors, ppd->vendor, models);
+		if (select) {
 			GtkTreeIter   iter;
 			GtkTreeModel *store = gtk_combo_box_get_model (
-				GTK_COMBO_BOX (glade_xml_get_widget (xml, "vendors")));
+				GTK_COMBO_BOX (glade_xml_get_widget (ds->xml, "vendors")));
 			gtk_list_store_append (GTK_LIST_STORE (store), &iter);
 			gtk_list_store_set (GTK_LIST_STORE (store), &iter,
 				0, ppd->vendor,
@@ -386,33 +409,50 @@ categorize_ppd (GCupsPPD *ppd, GHashTable *vendors, GladeXML *xml)
 	} else
 		drivers = g_hash_table_lookup (models, ppd->model);
 
-	if (xml != NULL && drivers == NULL) {
+	if (select && drivers == NULL) {
 		GtkTreeIter   iter;
 		GtkTreeModel *store = gtk_tree_view_get_model (
-			GTK_TREE_VIEW (glade_xml_get_widget (xml, "model_treeview")));
+			GTK_TREE_VIEW (glade_xml_get_widget (ds->xml, "model_treeview")));
 		gtk_list_store_append (GTK_LIST_STORE (store), &iter);
 		gtk_list_store_set (GTK_LIST_STORE (store), &iter,
 			0, ppd->model,
 			-1);
 	}
 
-	if (NULL != (dup = g_slist_find_custom (drivers, ppd, compare_models_by_driver))
-	    && dup->data != ppd) {
-		GCupsPPD const *existing = dup->data;
-		g_warning ("Two ppds have driver == '%s'\n\t->%s (%s) and\n\t->%s (%s)",
+	if (NULL != (tmp = g_slist_find_custom (drivers, ppd, compare_models_by_driver))
+	    && tmp->data != ppd) {
+		GCupsPPD *existing = tmp->data;
+
+		/* SuSE likes to tweak some of the ppds and install both copies.
+		 * Give the tweaked versions perference.  Drop stp variants in
+		 * preference to others */
+		if (NULL != strstr (ppd->filename, "suse.ppd") ||
+		    NULL != strstr (existing->filename, "stp/")) {
+			char *tmp = existing->filename;
+			existing->filename = ppd->filename;
+			ppd->filename = tmp;
+			ppd_free (ppd); /* prefer existing, drop the new one */
+			return;
+		}
+		if (NULL != strstr (existing->filename, "suse.ppd") ||
+		    NULL != strstr (ppd->filename, "stp/")) {
+			ppd_free (ppd); /* prefer existing, drop the new one */
+			return;
+		}
+		g_warning ("Two ppds have driver == '%s'\n\t->%s (%s[%d]) and\n\t->%s (%s)[%d]",
 			   ppd->driver,
-			   ppd->filename,	ppd->nickname,
-			   existing->filename,	existing->nickname);
+			   ppd->filename,	ppd->nickname, ppd->is_recommended,
+			   existing->filename,	existing->nickname, existing->is_recommended);
 	}
 
 	drivers = g_slist_insert_sorted (drivers, ppd, compare_models_by_driver);
 	g_hash_table_steal (models, ppd->model);
 	g_hash_table_insert (models, ppd->model, drivers);
 
-	if (xml != NULL) {
+	if (select) {
 		GdkPixbuf    *pixbuf;
 		GtkTreeIter   iter;
-		GtkComboBox  *combo = GTK_COMBO_BOX (glade_xml_get_widget (xml, "drivers"));
+		GtkComboBox  *combo = GTK_COMBO_BOX (glade_xml_get_widget (ds->xml, "drivers"));
 		GtkTreeModel *store = gtk_combo_box_get_model (combo);
 		gtk_list_store_append (GTK_LIST_STORE (store), &iter);
 		if (ppd->is_recommended)
@@ -427,13 +467,6 @@ categorize_ppd (GCupsPPD *ppd, GHashTable *vendors, GladeXML *xml)
 			3, pixbuf,
 			-1);
 	}
-}
-
-static void
-setup_ppd (GCupsDriverSelector *ds, GCupsPPD *ppd, gboolean select)
-{
-	ppd_extract_model_and_driver (ppd);
-	categorize_ppd (ppd, ds->vendors, select ? ds->xml : NULL);
 	g_hash_table_insert (ds->ppds, ppd->nickname, ppd);
 }
 
@@ -508,12 +541,13 @@ get_detected_ppd (GCupsDriverSelector *ds, char const *nickname)
 	vendor = NULL;
 	model = NULL;
 	vendors = vendor_list (ds->vendors);
-	for (ptr = vendors; ptr != NULL && model == NULL; ptr = ptr->next)
-		model = remove_vendor (ptr->data, nickname);
-	if (model != NULL)
-		vendor = g_strdup (ptr->data);
+	for (ptr = vendors; ptr != NULL ; ptr = ptr->next)
+		if (NULL != (model = remove_vendor (ptr->data, nickname))) {
+			vendor = g_strdup (ptr->data);
+			break;
+		}
 	g_slist_free (vendors);
-	if (vendor == NULL)
+	if (NULL == ptr)
 		return NULL;
 
 	/* Now we have a valid make and model.  Find the closest ppd */
@@ -720,9 +754,11 @@ static void
 cb_driver_changed (GCupsDriverSelector *ds)
 {
 	/* some debug spew */
+#if 1
 	GCupsPPD const *ppd = gcups_driver_selector_get (ds);
 	if (ppd != NULL)
 		g_print ("Selected ppd file = %s\n", ppd->filename);
+#endif
 
 	g_signal_emit (G_OBJECT (ds), signals [CHANGED], 0);
 }
